@@ -2,7 +2,7 @@
  Copyright (C) 1999-2004 IC & S  dbmail@ic-s.nl
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2025 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -68,6 +68,7 @@ typedef enum {
 } Driver_T;
 
 typedef enum {
+	DM_OVERQUOTA 	= -2, // over quota
 	DM_EQUERY 	= -1,
 	DM_SUCCESS 	= 0,
 	DM_EGENERAL 	= 1
@@ -373,7 +374,7 @@ typedef struct {
 	char hostname[64];
 	char *apop_stamp;		/**< timestamp for APOP */
 
-	uint64_t useridnr;			/**< Used by timsieved */
+	uint64_t useridnr;			/**< Used by sieved */
 	uint64_t totalsize;		/**< total size of messages */
 	uint64_t virtual_totalsize;
 	uint64_t totalmessages; 		/**< number of messages */
