@@ -93,6 +93,7 @@ ImapSession * dbmail_imap_session_new(Mempool_T pool)
 	ImapSession * self;
 	Field_T val;
 	gboolean login_disabled = TRUE;
+	gboolean enable_cram_md5 = TRUE;
 
 	self = mempool_pop(pool, sizeof(ImapSession));
 
@@ -109,11 +110,16 @@ ImapSession * dbmail_imap_session_new(Mempool_T pool)
 	if (SMATCH(val, "no"))
 		login_disabled = FALSE;
 
+	GETCONFIGVALUE("enable_cram_md5", "IMAP", val);
+	if (SMATCH(val, "no"))
+		enable_cram_md5 = FALSE;
+
 	self->state = CLIENTSTATE_NON_AUTHENTICATED;
 	self->args = mempool_pop(self->pool, sizeof(String_T) * MAX_ARGS);
 	self->fi = mempool_pop(self->pool, sizeof(fetch_items));
 	self->capa = Capa_new(self->pool);
 	self->preauth_capa = Capa_new(self->pool);
+
 	Capa_remove(self->preauth_capa, "ACL");
 	Capa_remove(self->preauth_capa, "RIGHTS=texk");
 	Capa_remove(self->preauth_capa, "NAMESPACE");
@@ -129,29 +135,28 @@ ImapSession * dbmail_imap_session_new(Mempool_T pool)
 	Capa_remove(self->preauth_capa, "ENABLE");
 	Capa_remove(self->preauth_capa, "QRESYNC");
 
-	if (! (server_conf && server_conf->ssl))
-		Capa_remove(self->preauth_capa, "STARTTLS");
+	Capa_remove(self->capa, "STARTTLS");
+	Capa_remove(self->capa, "LOGINDISABLED");
+	Capa_remove(self->capa, "AUTH=LOGIN");
+	Capa_remove(self->capa, "AUTH=PLAIN");
+	Capa_remove(self->capa, "AUTH=CRAM-MD5");
 
-	if (! Capa_match(self->preauth_capa, "STARTTLS"))
+	if (! (server_conf && server_conf->ssl)) {
+		Capa_remove(self->preauth_capa, "STARTTLS");
 		login_disabled = FALSE;
+	}
+
+	if ((! enable_cram_md5) || MATCH(db_params.authdriver, "LDAP")) {
+		Capa_remove(self->preauth_capa, "AUTH=CRAM-MD5");
+	}
 
 	if (login_disabled) {
-		if (! Capa_match(self->preauth_capa, "LOGINDISABLED"))
-			Capa_add(self->preauth_capa, "LOGINDISABLED");
 		Capa_remove(self->preauth_capa, "AUTH=LOGIN");
+		Capa_remove(self->preauth_capa, "AUTH=PLAIN");
 		Capa_remove(self->preauth_capa, "AUTH=CRAM-MD5");
 	} else {
 		Capa_remove(self->preauth_capa, "LOGINDISABLED");
 	}
-	if (MATCH(db_params.authdriver, "LDAP")) {
-		Capa_remove(self->capa, "AUTH=CRAM-MD5");
-		Capa_remove(self->preauth_capa, "AUTH=CRAM-MD5");
-	}
-
-	if (! (server_conf && server_conf->ssl)) {
-		Capa_remove(self->capa, "STARTTLS");
-		Capa_remove(self->preauth_capa, "STARTTLS");
-	}
 
 	self->physids = g_tree_new((GCompareFunc)ucmp);
 	self->mbxinfo = g_tree_new_full((GCompareDataFunc)ucmpdata,NULL,(GDestroyNotify)uint64_free,(GDestroyNotify)mailboxstate_destroy);
@@ -160,6 +165,29 @@ ImapSession * dbmail_imap_session_new(Mempool_T pool)
 	return self;
 }
 
+void dbmail_imap_session_encrypted(ImapSession *self)
+{
+	Field_T val;
+	gboolean enable_cram_md5 = TRUE;
+
+	GETCONFIGVALUE("enable_cram_md5", "IMAP", val);
+	if (SMATCH(val, "no"))
+		enable_cram_md5 = FALSE;
+
+	Capa_remove(self->preauth_capa, "STARTTLS");
+	Capa_remove(self->preauth_capa, "LOGINDISABLED");
+
+	if (! Capa_match(self->preauth_capa, "AUTH=LOGIN"))
+		Capa_add(self->preauth_capa, "AUTH=LOGIN");
+	if (! Capa_match(self->preauth_capa, "AUTH=PLAIN"))
+		Capa_add(self->preauth_capa, "AUTH=PLAIN");
+
+	if (enable_cram_md5 && (! MATCH(db_params.authdriver, "LDAP"))) {
+		if (! Capa_match(self->preauth_capa, "AUTH=CRAM-MD5"))
+			Capa_add(self->preauth_capa, "AUTH=CRAM-MD5");
+	}
+}
+
 static uint64_t dbmail_imap_session_message_load(ImapSession *self)
 {
 	TRACE(TRACE_DEBUG, "Call: dbmail_imap_session_message_load");
@@ -178,7 +206,7 @@ static uint64_t dbmail_imap_session_message_load(ImapSession *self)
 		}else{
 			/* previous behavior, a query will be performed in db */
 			if ((db_get_physmessage_id(self->msg_idnr, id)) != DM_SUCCESS) {
-				TRACE(TRACE_ERR,"can't find physmessage_id for message_idnr [%" PRIu64 "]", self->msg_idnr);
+				TRACE(TRACE_ERR,"[%p] can't find physmessage_id for message_idnr [%" PRIu64 "]", self, self->msg_idnr);
 				mempool_push(self->pool, id, sizeof(uint64_t));
 				g_free(id);
 				return 0;
@@ -207,7 +235,7 @@ static uint64_t dbmail_imap_session_message_load(ImapSession *self)
 	}
 
 	if (! self->message) {
-		TRACE(TRACE_ERR,"message retrieval failed");
+		TRACE(TRACE_ERR,"[%p] message retrieval failed", self);
 		return 0;
 	}
 
@@ -391,7 +419,7 @@ static int _imap_session_fetch_parse_partspec(ImapSession *self)
 	TRACE(TRACE_DEBUG,"token [%s], nexttoken [%s]", token, nexttoken);
 
 	for (j = 0; token[j]; j++) {
-		if (isdigit(token[j])) {
+		if (isdigit((unsigned char) token[j])) {
 			indigit = 1;
 			continue;
 		} else if (token[j] == '.') {
@@ -495,7 +523,7 @@ static int _imap_session_fetch_parse_octet_range(ImapSession *self)
 				if (delimpos != -1) 
 					return -2;
 				delimpos = j;
-			} else if (!isdigit (token[j]))
+			} else if (!isdigit ((unsigned char) token[j]))
 				return -2;
 		}
 		if (delimpos == -1 || delimpos == 1 || delimpos == (int) (strlen(token) - 2))
@@ -523,16 +551,32 @@ static int _imap_session_fetch_parse_octet_range(ImapSession *self)
 	return 0;	/* DONE */
 }
 
-
-#define TOKENAT(a) (self->args[self->args_idx+a]?p_string_str(self->args[self->args_idx+a]):NULL)
-#define NEXTTOKEN TOKENAT(1)
-#define TOKEN TOKENAT(0)
+/**
+ * Tokens are stored in a c array accessed via a pointer
+ * that doesn't know about the array.
+ * Testing for the existence of an element causes an Invalid read
+ * TODO convert ImapSession->args into a gtk GArray
+ */
+const char * token_first(ImapSession * self) {
+	if (self->args[self->args_idx]) {
+		return p_string_str(self->args[self->args_idx]);
+	} else {
+		return NULL;
+	}
+}
+const char * token_next(ImapSession * self) {
+	if (self->args[self->args_idx+1]) {
+		return p_string_str(self->args[self->args_idx+1]);
+	} else {
+		return NULL;
+	}
+}
 
 int dbmail_imap_session_fetch_parse_args(ImapSession * self)
 {
 	int ispeek = 0;
-	const char *token = TOKEN;
-	const char *nexttoken = NEXTTOKEN;
+	const char *token = token_first(self);
+	const char *nexttoken = token_next(self);
 
 	if (!token) return -1; // done
 	if ((token[0] == ')') && (! nexttoken)) return -1; // done
@@ -575,8 +619,8 @@ int dbmail_imap_session_fetch_parse_args(ImapSession * self)
 
 		if (MATCH(token,"body.peek")) ispeek=1;
 		
-		nexttoken = NEXTTOKEN;
-		
+		nexttoken = token_next(self);
+
 		if (! nexttoken || ! MATCH(nexttoken,"[")) {
 			if (ispeek) return -2;	/* error DONE */
 			self->fi->msgparse_needed = 1;
@@ -587,8 +631,8 @@ int dbmail_imap_session_fetch_parse_args(ImapSession * self)
 			self->args_idx++;	/* now pointing at '[' (not the last arg, parentheses are matched) */
 			self->args_idx++;	/* now pointing at what should be the item type */
 
-			token = TOKEN;
-			nexttoken = NEXTTOKEN;
+			token = token_first(self);
+			nexttoken = token_next(self);
 
 			TRACE(TRACE_DEBUG,"[%p] token [%s], nexttoken [%s]", self, token, nexttoken);
 
@@ -886,7 +930,7 @@ static void _fetch_headers(ImapSession *self, body_fetch *bodyfetch, gboolean no
 
 				/* Build content as Header: value \n */
 				fld2 = g_strdup_printf("%s", fld);
-				fld2[0] = toupper(fld[0]);
+				fld2[0] = toupper((unsigned char) fld[0]);
 				new = g_strdup_printf("%s%s: %s\n", old?old:"", fld2, val);
 				g_free(val);
 				g_free(fld2);
@@ -1395,8 +1439,8 @@ int dbmail_imap_session_handle_auth(ImapSession * self, const char * username, c
 				char *enctype = NULL;
 				if (userid) enctype = auth_getencryption(userid);
 				if ((! enctype) || (! MATCH(enctype,""))) {
-					Capa_remove(self->capa,"AUTH=CRAM-MD5");
-					dbmail_imap_session_buff_printf(self, "* CAPABILITY %s\r\n", Capa_as_string(self->capa));
+					Capa_remove(self->preauth_capa,"AUTH=CRAM-MD5");
+					dbmail_imap_session_buff_printf(self, "* CAPABILITY %s\r\n", Capa_as_string(self->preauth_capa));
 				}
 				if (enctype) g_free(enctype);
 			}
@@ -1788,9 +1832,6 @@ int dbmail_imap_session_set_state(ImapSession *self, ClientState_T state)
 			assert(self->ci);
 			TRACE(TRACE_DEBUG,"[%p] set timeout to [%d]", self, server_conf->timeout);
 			self->ci->timeout.tv_sec = server_conf->timeout; 
-			Capa_remove(self->capa, "AUTH=login");
-			Capa_remove(self->capa, "AUTH=CRAM-MD5");
-
 			break;
 
 		default:
@@ -2014,6 +2055,32 @@ int imap4_tokenizer_main(ImapSession *self, const char *buffer)
 				dbmail_imap_session_prompt(self,"password");
 				return 0;
 			}
+
+		} else if (MATCH(p_string_str(self->args[0]),"PLAIN")) {
+			uint64_t len;
+			char *lnul, *rnul, *tmp;
+
+			tmp = (char *) g_base64_decode(s, &len);
+			if (! tmp) {
+				return -1;
+			}
+			tmp = (char *) g_realloc(tmp, len+1);
+			tmp[len] = '\0';
+
+			lnul = (char *) memchr(tmp, 0, len);
+			rnul = (char *) memrchr(tmp, 0, len);
+
+			if (lnul && rnul && (lnul != rnul) && (rnul-lnul > 1) && (rnul-tmp < (long int) len)) {
+				if ((lnul == tmp) || (strcmp(tmp, lnul+1) == 0)) {
+					self->args[self->args_idx++] = p_string_new(self->pool, lnul+1);
+					self->args[self->args_idx++] = p_string_new(self->pool, rnul+1);
+					g_free(tmp);
+					goto finalize;
+				}
+			}
+			g_free(tmp);
+			return -1;
+
 		} else if (MATCH(p_string_str(self->args[0]),"CRAM-MD5")) {
 			if (self->args_idx == 1) {
 				/* decode and store the response */
@@ -2151,11 +2218,15 @@ int imap4_tokenizer_main(ImapSession *self, const char *buffer)
 finalize:
 	if (self->args_idx == 1) {
 		if (Capa_match(self->preauth_capa, "AUTH=LOGIN") && MATCH(p_string_str(self->args[0]),"LOGIN")) {
-			TRACE(TRACE_DEBUG, "[%p] prompt for authenticate tokens", self);
-
+			TRACE(TRACE_DEBUG, "[%p] prompt for LOGIN authenticate tokens", self);
 			/* ask for username */
 			dbmail_imap_session_prompt(self,"username");
 			return 0;
+		} else if (Capa_match(self->preauth_capa, "AUTH=PLAIN") && MATCH(p_string_str(self->args[0]),"PLAIN")) {
+			TRACE(TRACE_DEBUG, "[%p] prompt for PLAIN authentication string", self);
+			/* ask for base64 encoded authentication string */
+			dbmail_imap_session_prompt(self,"");
+			return 0;
 		} else if (Capa_match(self->preauth_capa, "AUTH=CRAM-MD5") && MATCH(p_string_str(self->args[0]),"CRAM-MD5")) {
 			const gchar *s;
 			gchar *t;
