@@ -83,7 +83,7 @@ typedef struct {
 ImapSession * dbmail_imap_session_new(Mempool_T);
 ImapSession * dbmail_imap_session_set_command(ImapSession * self, const char * command);
 
-void dbmail_imap_session_reset(ImapSession *session);
+void dbmail_imap_session_encrypted(ImapSession *session);
 
 void dbmail_imap_session_args_free(ImapSession *self, gboolean all);
 void dbmail_imap_session_fetch_free(ImapSession *self, gboolean all);
@@ -111,5 +111,8 @@ int imap4_tokenizer_main(ImapSession *self, const char *buffer);
 
 void _ic_cb_leave(gpointer data);
 
+const char * token_first(ImapSession * self);
+const char * token_next(ImapSession * self);
+
 #endif
 
