@@ -1,7 +1,7 @@
 /*
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2025 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
@@ -59,8 +59,9 @@ DbmailMailbox * dbmail_mailbox_new(Mempool_T pool, uint64_t id) {
 static gboolean _node_free(GNode *node, gpointer data) {
 	DbmailMailbox *self = (DbmailMailbox *) data;
 	search_key *s = (search_key *) node->data;
-	if (s->found)
+	if (s->found) {
 		g_tree_destroy(s->found);
+	}
 	mempool_push(self->pool, s, sizeof (search_key));
 	return FALSE;
 }
@@ -68,8 +69,12 @@ static gboolean _node_free(GNode *node, gpointer data) {
 void dbmail_mailbox_free(DbmailMailbox *self) {
 	Mempool_T pool = self->pool;
 	gboolean freepool = self->freepool;
-	if (self->found) g_tree_destroy(self->found);
-	if (self->sorted) g_list_destroy(self->sorted);
+	if (self->found) {
+		g_tree_destroy(self->found);
+	}
+	if (self->sorted) {
+		g_list_free_full(g_steal_pointer (&self->sorted), g_free);
+	}
 	if (self->search) {
 		g_node_traverse(g_node_get_root(self->search), G_POST_ORDER, G_TRAVERSE_ALL, -1, (GNodeTraverseFunc) _node_free, self);
 		g_node_destroy(self->search);
@@ -413,7 +418,6 @@ char * dbmail_mailbox_imap_modseq_as_string(DbmailMailbox *self, gboolean uid) {
 		return s;
 	}
 
-	t = g_string_new("");
 	if (uid || dbmail_mailbox_get_uid(self)) {
 		l = g_tree_keys(self->found);
 	} else {
@@ -441,8 +445,10 @@ char * dbmail_mailbox_imap_modseq_as_string(DbmailMailbox *self, gboolean uid) {
 		l = g_list_next(l);
 	}
 
+	l = g_list_first(l);
 	g_list_free(l);
 
+	t = g_string_new("");
 	if (self->modseq)
 		g_string_append_printf(t, " (MODSEQ %" PRIu64 ")", maxseq);
 
@@ -466,7 +472,7 @@ char * dbmail_mailbox_ids_as_string(
 	TRACE(TRACE_DEBUG, "Call: dbmail_mailbox_ids_as_string");
 	GString *t;
 	gchar *s = NULL;
-	GList *l = NULL, *h = NULL;
+	GList *l = NULL;
 	GTree *msginfo;
 	GTree *msn;
 	uint64_t maxseq = 0;
@@ -483,8 +489,6 @@ char * dbmail_mailbox_ids_as_string(
 		l = g_tree_values(self->found);
 	}
 
-	h = l;
-
 	msginfo = MailboxState_getMsginfo(self->mbstate);
 	msn = MailboxState_getMsn(self->mbstate);
 
@@ -508,13 +512,12 @@ char * dbmail_mailbox_ids_as_string(
 		l = g_list_next(l);
 	}
 
-	g_list_free(h);
+	l = g_list_first(l);
+	g_list_free(l);
 
-	s = t->str;
-	g_string_free(t, FALSE);
+	s = g_string_free(t, FALSE);
 
 	return g_strchomp(s);
-
 }
 
 char * dbmail_mailbox_sorted_as_string(DbmailMailbox *self) {
@@ -1192,6 +1195,7 @@ static gboolean _do_sort(GNode *node, DbmailMailbox *self) {
 	}
 
 	z = g_tree_new((GCompareFunc) ucmp);
+
 	c = db_con_get();
 	TRY
 	r = db_query(c, q->str);
