@@ -154,7 +154,8 @@ static int check_state_and_args(ImapSession * self, int minargs, int maxargs, Cl
 void _ic_capability_enter(dm_thread_data *D)
 {
 	SESSION_GET;
-	dbmail_imap_session_buff_printf(self, "* %s %s\r\n", self->command, Capa_as_string(self->capa));
+	dbmail_imap_session_buff_printf(self, "* %s %s\r\n", self->command,
+		Capa_as_string(self->state == CLIENTSTATE_NON_AUTHENTICATED ? self->preauth_capa : self->capa));
 	SESSION_OK;
 	SESSION_RETURN;
 }
@@ -180,13 +181,13 @@ int _ic_starttls(ImapSession *self)
 	}
 	ci_write(self->ci, "%s OK Begin TLS now\r\n", self->tag);
 	i = ci_starttls(self->ci);
-	
-	Capa_remove(self->capa, "STARTTLS");
-	Capa_remove(self->capa, "LOGINDISABLED");
-	
+
 	if (i < 0) i = 0;
 
-	if (i == 0) return 3; /* done */
+	if (i == 0) {
+		dbmail_imap_session_encrypted(self);
+		return 3; /* done */
+	}
 
 	return i;
 }
@@ -298,9 +299,10 @@ int _ic_authenticate(ImapSession *self)
 {
 	if (self->command_type == IMAP_COMM_AUTH) {
 		if (!check_state_and_args(self, 1, 3, CLIENTSTATE_NON_AUTHENTICATED)) return 1;
-		/* check authentication method */
-		if ( (! MATCH(p_string_str(self->args[self->args_idx]), "login")) && \
-			(! MATCH(p_string_str(self->args[self->args_idx]), "cram-md5")) ) {
+		/* check authentication method and announced support */
+		if ( (! (MATCH(p_string_str(self->args[self->args_idx]), "login") && Capa_match(self->preauth_capa, "AUTH=LOGIN"))) &&
+		     (! (MATCH(p_string_str(self->args[self->args_idx]), "plain") && Capa_match(self->preauth_capa, "AUTH=PLAIN"))) &&
+		     (! (MATCH(p_string_str(self->args[self->args_idx]), "cram-md5") && Capa_match(self->preauth_capa, "AUTH=CRAM-MD5"))) ) {
 			dbmail_imap_session_buff_printf(self, "%s NO Invalid authentication mechanism specified\r\n", self->tag);
 			return 1;
 		}
@@ -2235,7 +2237,7 @@ static gboolean _do_store(uint64_t *id, gpointer UNUSED value, dm_thread_data *D
 		if (cmd_store_silent==1 && changed){
 			showflags = showflags || changed;
 		}
-		TRACE(TRACE_INFO,"requested FLAGS.SILENT but a change was detected into mailbox and command_store_flags_silent_ignore_silent=%d so ignore FLAGS.SILENT and return message information with flags",cmd_store_silent);
+		TRACE(TRACE_INFO,"[%p] requested FLAGS.SILENT but a change was detected into mailbox and command_store_flags_silent_ignore_silent=%d so ignore FLAGS.SILENT and return message information with flags", self, cmd_store_silent);
 
 		_fetch_update(self, msginfo, showmodseq, showflags);
 	}
@@ -2457,7 +2459,7 @@ static gboolean _do_copy(uint64_t *id, gpointer UNUSED value, ImapSession *self)
 	int result;
 	uint64_t *new_ids_element = NULL;
 	if (!g_tree_lookup(self->mailbox->mbstate->msginfo, id)){
-		TRACE(TRACE_WARNING,"Copy message [%ld] failed security issue, trying to copy message that are not in this mailbox",*id);
+		TRACE(TRACE_WARNING,"[%p] Copy message [%ld] failed security issue, trying to copy message that are not in this mailbox", self, *id);
 		//dbmail_imap_session_buff_printf(self, "%s NO security issue, trying to copy message that are not in this mailbox\r\n",self->tag);
 		return FALSE;
 	}
@@ -2465,15 +2467,15 @@ static gboolean _do_copy(uint64_t *id, gpointer UNUSED value, ImapSession *self)
 	db_message_set_seq(*id, cmd->seq);
 	if (result == -1) {
 		/* uid not found, according to RFC 3501 section 6.4.8, should continue  */
-		TRACE(TRACE_WARNING,"Copy message [%ld] failed due to missing in database. continue.",*id);
+		TRACE(TRACE_WARNING,"[%p] Copy message [%ld] failed due to missing in database. continue.", self, *id);
 		/* continue operation, do not close or send various info on connection */
 		//dbmail_imap_session_buff_printf(self, "* BYE internal dbase error\r\n");
 		//return TRUE;
 		return FALSE;
 	}
 	if (result == -2) {
-		TRACE(TRACE_WARNING,"Copy message [%ld] failed due to `%s NO quotum would exceed`",*id,self->tag);
-		dbmail_imap_session_buff_printf(self, "%s NO quotum would exceed\r\n", self->tag);
+		TRACE(TRACE_WARNING,"[%p] Copy message [%ld] failed due to `%s NO quota would be exceeded`", self, *id, self->tag);
+		dbmail_imap_session_buff_printf(self, "%s NO quota would be exceeded\r\n", self->tag);
 		return TRUE;
 	}
 	// insert the new uid to the new_ids collection
