@@ -2,7 +2,7 @@
  Copyright (C) 1999-2004 IC & S  dbmail@ic-s.nl
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2025 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -38,7 +38,7 @@ extern DBParam_T db_params;
 #define U URL_T
 
 
-int auth_connect()
+int auth_connect(void)
 {
 	/* this function is only called after a connection has been made
 	 * if, in the future this is not the case, db.h should export a 
@@ -47,7 +47,7 @@ int auth_connect()
 	return 0;
 }
 
-int auth_disconnect()
+int auth_disconnect(void)
 {
 	return 0;
 }
@@ -158,8 +158,10 @@ char *auth_getencryption(uint64_t user_idnr)
 	return res;
 }
 
-
-static GList *user_get_deliver_to(const char *username)
+/**
+ * Given an email address, return the deliver_to
+ */
+static GList *user_get_deliver_to(const char *address)
 {
 	INIT_QUERY;
 	C c; R r; S s;
@@ -174,7 +176,7 @@ static GList *user_get_deliver_to(const char *username)
 	c = db_con_get();
 	TRY
 		s = db_stmt_prepare(c, query);
-		db_stmt_set_str(s, 1, username);
+		db_stmt_set_str(s, 1, address);
 
 		r = db_stmt_query(s);
 		while (db_result_next(r))
@@ -188,7 +190,6 @@ static GList *user_get_deliver_to(const char *username)
 	return d;
 }
 
-
 int auth_check_user_ext(const char *username, GList **userids, GList **fwds, int checks)
 {
 	int occurences = 0;
@@ -207,7 +208,7 @@ int auth_check_user_ext(const char *username, GList **userids, GList **fwds, int
 
 	if (! d) {
 		if (checks == 0) {
-			TRACE(TRACE_DEBUG, "user %s not in aliases table", username);
+			TRACE(TRACE_DEBUG, "user [%s] not in aliases table", username);
 			return 0;
 		}
 		/* found the last one, this is the deliver to
