@@ -2,7 +2,7 @@
  Copyright (C) 2003 Aaron Stone
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2025 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -28,19 +28,6 @@
 #define THIS_MODULE "sievecmd"
 #define PNAME "dbmail/sievecmd"
 
-/* Loudness and assumptions. */
-int yes_to_all = 0;
-int no_to_all = 0;
-int verbose = 0;
-/* Don't be helpful. */
-int quiet = 0;
-/* Don't print errors. */
-int reallyquiet = 0;
-
-#define qverbosef(fmt, args...) if (verbose) printf(fmt, ##args)
-#define qprintf(fmt, args...) if (! (quiet||reallyquiet)) printf(fmt, ##args) 
-#define qerrorf(fmt, args...) if (! reallyquiet) fprintf(stderr, fmt, ##args) 
-
 extern char configFile[PATH_MAX];
 
 static int do_showhelp(void);
@@ -49,12 +36,54 @@ static int do_activate(uint64_t user_idnr, char *name);
 static int do_deactivate(uint64_t user_idnr, char *name);
 static int do_remove(uint64_t user_idnr, char *name);
 static int do_edit(uint64_t user_idnr, char *name);
-static int do_insert(uint64_t user_idnr, char *name, char *source);
+static int do_import(uint64_t user_idnr, char *name, char *source);
 static int do_cat(uint64_t user_idnr, char *name, FILE *out);
 
+int do_showhelp(void)
+{
+	printf(
+	"*** dbmail-sievecmd ***\n"
+//	Try to stay under the standard 80 column width
+//	0........10........20........30........40........50........60........70........80
+	"Use this program to manage your users' Sieve scripts.\n"
+	"See the man page for more info. Summary:\n\n"
+	"  -u, --user username        Username of script user \n"
+	"  -l, --list                 List scripts belonging to user \n"
+	"  -a, --activate scriptname  Activate the named script \n"
+	"                             (only one script can be active; \n"
+	"                             deactivates any others) \n"
+	"  -d, --deactivate scriptname Deactivate the named script \n"
+	"  -c, --show scriptname      Show the contents of the named script\n"
+	"  -e, --edit scriptname      Edit the contents of the named script\n"
+	"                             (if no script is given, the active \n"
+	"                             script is shown) \n"
+	"  -i, --import file          Insert the named script from file \n"
+	"                             (a single dash, -, reads input \n"
+	"                             from STDIN) \n"
+	"  -r, --remove scriptname    Remove the named script \n"
+	"                             (if script was active, no script is \n"
+	"                             active after deletion) \n"
+
+	"\nCommon options for all DBMail utilities:\n"
+	"  -f, --config file  specify an alternative config file\n"
+	"                     Default: %s\n"
+	"  -q, --quiet        quietly skip interactive prompts\n"
+	"                     use twice to suppress error messages\n"
+	"  -n, --no           show the intended action but do not perform it, no to all\n"
+	"  -y, --yes          perform all proposed actions, as though yes to all\n"
+	"  -v, --verbose      verbose details\n"
+	"  -V, --version      show the version\n"
+	"  -h, --help         show this help message\n"
+	, configFile);
+
+	return 0;
+}
+
 int main(int argc, char *argv[])
 {
-	int res = 0, opt = 0, opt_prev = 0;
+	int opt = 0;
+	int option_index = 0;
+	int res = 0;
 	uint64_t user_idnr = 0;
 	char *user_name = NULL;
 	char *script_name = NULL;
@@ -62,7 +91,7 @@ int main(int argc, char *argv[])
 	extern char *optarg;
 	extern int opterr;
 
-	int activate = 0, deactivate = 0, insert = 0;
+	int activate = 0, deactivate = 0, import = 0;
 	int remove = 0, list = 0, cat = 0, help = 0, edit = 0;
 
 	config_get_file();
@@ -70,15 +99,31 @@ int main(int argc, char *argv[])
 	setvbuf(stdout, 0, _IONBF, 0);
 
 	/* get options */
-	opterr = 0;		/* suppress error message from getopt() */
-	while ((opt = getopt(argc, argv,
-		"-a::d::i:c::r:u:le::" /* Major modes */
-		/*"i"*/ "f:qnyvVh" /* Common options */ )) != -1) {
-		/* The initial "-" of optstring allows unaccompanied
-		 * options and reports them as the optarg to opt 1 (not '1') */
-		if (opt == 1)
-			opt = opt_prev;
-		opt_prev = opt;
+	static struct option long_options[] = {
+		{"user", required_argument, NULL, 'u'},
+		{"list", no_argument, NULL, 'l'},
+		{"activate", required_argument, NULL, 'a'},
+		{"deactivate", required_argument, NULL, 'd'},
+		{"show", required_argument, NULL, 'c'},
+		{"edit", required_argument, NULL, 'e'},
+		{"import", required_argument, NULL, 'i'},
+		{"remove", required_argument, NULL, 'r'},
+
+		{"config",    required_argument, NULL, 'f'},
+		{"quiet",     no_argument, NULL, 'q'},
+		{"no",        no_argument, NULL, 'n'},
+		{"yes",       no_argument, NULL, 'y'},
+		{"help",      no_argument, NULL, 'h'},
+		{"verbose",   no_argument, NULL, 'v'},
+		{"version",   no_argument, NULL, 'V'},
+		{NULL,        0,           NULL, 0}
+	};
+	/* Check for commandline options. */
+	opterr = 0; /* suppress error message from getopt_log() */
+	while ((opt = getopt_long(argc, argv,
+		"a::d::i:c::r:u:le::" /* Major modes */
+		"f:qvVh", /* Common options */
+		long_options, &option_index)) != -1) {
 
 		switch (opt) {
 		case -1:
@@ -90,8 +135,8 @@ int main(int argc, char *argv[])
 		case 'd': /* deactivate */
 			deactivate = 1;
 			goto major_script;
-		case 'i': /* insert */
-			insert = 1;
+		case 'i': /* import */
+			import = 1;
 			goto major_script;
 		case 'r': /* remove */
 			remove = 1;
@@ -132,7 +177,7 @@ int main(int argc, char *argv[])
 			break;
 
 		/* Common options */
-		/*case 'i': FIXME: this is from user.c, but we're using -i for insertion.
+		/*case 'i': FIXME: this is from user.c, but we're using -i for importing.
 			printf("Interactive console is not supported in this release.\n");
 			return 1;*/
 
@@ -141,7 +186,7 @@ int main(int argc, char *argv[])
 				memset(configFile, 0, sizeof(configFile));
 				strncpy(configFile, optarg, sizeof(configFile)-1);
 			} else {
-				qerrorf("dbmail-users: -f requires a filename\n\n");
+				qprintf("dbmail-users: -f requires a filename\n\n");
 				return 1;
 			}
 			break;
@@ -184,7 +229,7 @@ int main(int argc, char *argv[])
 	}
 
 /*
-	if (insert) printf("got insert\n");
+	if (import) printf("got import\n");
 	if (remove) printf("got remove\n");
 	if (list) printf("got list\n");
 	if (cat) printf("got cat\n");
@@ -193,57 +238,59 @@ int main(int argc, char *argv[])
 */
 
 	/* Only one major mode is allowed */
-	if ((edit + insert + remove + list + cat > 1)
-	/* Only insert/edit are allowed together with activate or deactivate */
+	if ((edit + import + remove + list + cat > 1)
+	/* Only import/edit are allowed together with activate or deactivate */
 	 || ((remove + list + cat == 1) && (activate + deactivate > 0))
 	/* You may either activate or deactivate as a mode on its own*/
-	 || (((edit + insert + remove + list + cat == 0) && (activate + deactivate != 1)))
+	 || (((edit + import + remove + list + cat == 0) && (activate + deactivate != 1)))
 	 || (help || !user_name || (no_to_all && yes_to_all))) {
 		do_showhelp();
 		goto mainend;
 	}
 
 	/* read the config file */
-        if (config_read(configFile) == -1) {
-                qerrorf("Failed. Unable to read config file %s\n",
-                        configFile);
-                res = -1;
-                goto mainend;
-        }
+	if (config_read(configFile) == -1) {
+		qprintf("Failed. Unable to read config file %s\n",
+				configFile);
+		res = -1;
+		goto mainend;
+	}
                 
 	SetTraceLevel("DBMAIL");
 	GetDBParams();
 
+	TRACE(TRACE_DEBUG,"Hello [%s]", configFile);
+
 	/* Open database connection */
 	if (db_connect() != 0) {
-		qerrorf("Failed. Could not connect to database (check log)\n");
+		qprintf("Failed. Could not connect to database (check log)\n");
 		g_free(user_name);
 		return -1;
 	}
 
 	/* Open authentication connection */
 	if (auth_connect() != 0) {
-		qerrorf("Failed. Could not connect to authentication (check log)\n");
+		qprintf("Failed. Could not connect to authentication (check log)\n");
 		g_free(user_name);
 		return -1;
 	}
 
 	/* Retrieve the user ID number */
 	if (! auth_user_exists(user_name, &user_idnr)) {
-		qerrorf("User [%s] does not exist!\n", user_name);
+		qprintf("User [%s] does not exist!\n", user_name);
 		res = -1;
 		goto mainend;
 	}
 
-	if (insert)
-		res = do_insert(user_idnr, script_name, script_source);
+	if (import)
+		res = do_import(user_idnr, script_name, script_source);
 	if (remove)
 		res = do_remove(user_idnr, script_name);
 	if (edit)
 		res = do_edit(user_idnr, script_name);
 	if (activate)
-		/* Don't activate the script if it wasn't inserted/edited */
-		if (!(insert && res) && !(edit && res))
+		/* Don't activate the script if it wasn't imported/edited */
+		if (!(import && res) && !(edit && res))
 			res = do_activate(user_idnr, script_name);
 	if (deactivate)
 		res = do_deactivate(user_idnr, script_name);
@@ -268,12 +315,12 @@ int main(int argc, char *argv[])
 int do_activate(uint64_t user_idnr, char *name)
 {
 	if (!name) {
-		qerrorf("Must give the name of a script to activate.\n");
+		qprintf("Must give the name of a script to activate.\n");
 		return -1;
 	}
 
 	if (! dm_sievescript_activate(user_idnr, name)) {
-		qerrorf("Error activating script [%s].\n"
+		qprintf("Error activating script [%s].\n"
 		       "It is possible that no script is currently active!\n",
 		       name);
 		return -1;
@@ -291,12 +338,12 @@ int do_deactivate(uint64_t user_idnr, char *name)
 
 	if (!name) {
 		if (dm_sievescript_get(user_idnr, &scriptname)) {
-			qerrorf("Database error when fetching active script.\n");
+			qprintf("Database error when fetching active script.\n");
 			return -1;
 		}
 		
 		if (scriptname == NULL) {
-			qerrorf("No active script found.\n");
+			qprintf("No active script found.\n");
 			return -1;
 		}
 
@@ -304,7 +351,7 @@ int do_deactivate(uint64_t user_idnr, char *name)
 	}
 
 	if (! dm_sievescript_deactivate(user_idnr, name)) {
-		qerrorf("Error deactivating script [%s].\n", name);
+		qprintf("Error deactivating script [%s].\n", name);
 		return -1;
 	}
 	qprintf("Script [%s] is now deactivated."
@@ -326,47 +373,52 @@ int do_edit(uint64_t user_idnr, char *name)
 	char *scriptname = NULL;
 	struct stat stat_before, stat_after;
 
+	TRACE(TRACE_DEBUG,"User [%lu] Script [%s]", user_idnr, name);
+
 	if (!name) {
 		if (dm_sievescript_get(user_idnr, &scriptname)) {
-			qerrorf("Database error when fetching active script!\n");
+			qprintf("Database error when fetching active script!\n");
 			ret = 1;
 			goto cleanup;
 		}
 		
 		if (scriptname == NULL) {
-			qerrorf("No active script found!\n");
+			qprintf("No active script found!\n");
 			ret = 1;
 			goto cleanup;
 		}
 
 		name = scriptname;
 	}
+	TRACE(TRACE_DEBUG,"Using script [%s]", scriptname);
 
 	/* Check for the EDITOR environment variable. */
 	editor = getenv("EDITOR");
 
 	if (!editor) {
-		qerrorf("No EDITOR environment variable.\n");
+		qprintf("No EDITOR environment variable.\n");
 		ret = 1;
 		goto cleanup;
 	}
 
 	/* Open a temp file. */
 	if (!(tmp = tempnam(NULL, "dbmail"))) {
-		qerrorf("Could not make temporary file name: %s\n", strerror(errno));
+		qprintf("Could not make temporary file name: %s\n", strerror(errno));
 		ret = 1;
 		goto cleanup;
 	}
+	TRACE(TRACE_DEBUG,"Using temp filename [%s]", tmp);
 
 	if (!(ftmp = fopen(tmp, "w+"))) {
-		qerrorf("Could not open temporary file [%s]: %s\n", tmp, strerror(errno));
+		qprintf("Could not open temporary file [%s]: %s\n", tmp, strerror(errno));
+		TRACE(TRACE_DEBUG,"Unable to open [%s]: %s", tmp, strerror(errno));
 		ret = 1;
 		goto cleanup;
 	}
 
 	/* do_cat the script. */
 	if (do_cat(user_idnr, name, ftmp)) {
-		qerrorf("Could not dump script [%s] to temporary file.\n", name);
+		qprintf("Could not dump script [%s] to temporary file.\n", name);
 		ret = 1;
 		goto cleanup;
 	}
@@ -378,29 +430,31 @@ int do_edit(uint64_t user_idnr, char *name)
 	/* Call the editor. */
 	editor_cmd = g_strdup_printf("%s %s", editor, tmp);
 	if ((editor_val = system(editor_cmd))) {
-		qerrorf("Execution of EDITOR [%s] returned non-zero [%d].\n", editor, editor_val);
+		qprintf("Execution of EDITOR [%s] returned non-zero [%d].\n", editor, editor_val);
+		TRACE(TRACE_DEBUG,"Execution of EDITOR [%s] returned non-zero [%d]", editor, editor_val);
 		ret = 1;
 		goto cleanup;
 	}
 
 	if (fstat(fileno(ftmp), &stat_after) == -1) {
 		int serr = errno;
-		qerrorf("Stat failed: [%s]", strerror(serr));
+		qprintf("Stat failed: [%s]", strerror(serr));
 		ret = 1;
 		goto cleanup;
 	}
 
-	/* If the file does not appear to have changed, cancel insertion. */
+	/* If the file does not appear to have changed, cancel importing. */
 	if ((stat_before.st_mtime == stat_after.st_mtime)
 	 && (stat_before.st_size == stat_after.st_size)) {
 		qprintf("File not modified, canceling.\n");
+		TRACE(TRACE_DEBUG,"File not modified, canceling");
 		ret = 0;
 		goto cleanup;
 	}
 
-	/* do_insert the script (set yes_to_all as we will overwrite). */
+	/* do_import the script (set yes_to_all as we will overwrite). */
 	yes_to_all = 1;
-	if (do_insert(user_idnr, name, tmp)) {
+	if (do_import(user_idnr, name, tmp)) {
 		char dbmail_invalid_file[] = "dbmail-invalid.xxx.sieve";
 		struct stat stat_inv;
 		int i;
@@ -409,13 +463,16 @@ int do_edit(uint64_t user_idnr, char *name)
 			sprintf(dbmail_invalid_file, "dbmail-invalid.%d.sieve", i);
 			if (stat(dbmail_invalid_file, &stat_inv)) {
 				/* Try to rename the tmp file to this unused name. */
-				if (rename(tmp, dbmail_invalid_file) == 0)
+				if (rename(tmp, dbmail_invalid_file) == 0) {
 					break;
-				else
-					qerrorf("Could not save script to [%s]: %s\n", dbmail_invalid_file, strerror(errno));
+				} else {
+					TRACE(TRACE_DEBUG,"Could not save script to [%s]: %s", dbmail_invalid_file, strerror(errno));
+					qprintf("Could not save script to [%s]: %s\n", dbmail_invalid_file, strerror(errno));
+				}
 			}
 		}
-		qerrorf("Saved script to [%s]\n", dbmail_invalid_file);
+		TRACE(TRACE_DEBUG,"Saved script to [%s]\n", dbmail_invalid_file);
+		qprintf("Saved script to [%s]\n", dbmail_invalid_file);
 	}
 
 	/* Ok, all done. */
@@ -442,24 +499,24 @@ int do_cat(uint64_t user_idnr, char *name, FILE *out)
 		res = dm_sievescript_get(user_idnr, &scriptname);
 
 	if (res != 0) {
-		qerrorf("Database error when fetching active script!\n");
+		qprintf("Database error when fetching active script!\n");
 		return -1;
 	}
 	
 	if (scriptname == NULL) {
-		qerrorf("No active script found!\n");
+		qprintf("No active script found!\n");
 		return -1;
 	}
 
 	res = dm_sievescript_getbyname(user_idnr, scriptname, &buf);
 
 	if (res != 0) {
-		qerrorf("Database error when fetching script!\n");
+		qprintf("Database error when fetching script!\n");
 		return -1;
 	}
 	
 	if (buf == NULL) {
-		qerrorf("Script not found!\n");
+		qprintf("Script not found!\n");
 		return -1;
 	}
 
@@ -518,7 +575,7 @@ static int read_from_stream(FILE * instream, char **m_buf, int maxlen)
         return 0;
 }
 
-int do_insert(uint64_t user_idnr, char *name, char *source)
+int do_import(uint64_t user_idnr, char *name, char *source)
 {
 	int res = 0, was_found = 0;
 	char *buf = NULL;
@@ -531,11 +588,11 @@ int do_insert(uint64_t user_idnr, char *name, char *source)
 		was_found = 1;
 	}
 	if (res != 0) {
-		qerrorf("Could not determine if a script by that name already exists.\n");
+		qprintf("Could not determine if a script by that name already exists.\n");
 		return -1;
 	}
 	if (was_found && !yes_to_all) {
-		qerrorf("A script by that name already exists. Use -y option to overwrite it.\n");
+		qprintf("A script by that name already exists. Use --yes option to overwrite it.\n");
 		return -1;
 	}
 
@@ -547,7 +604,7 @@ int do_insert(uint64_t user_idnr, char *name, char *source)
 		file = fopen(source, "r");
 
 	if (!file) {
-		qerrorf("Could not open file [%s]: %s\n",
+		qprintf("Could not open file [%s]: %s\n",
 			optarg, strerror(errno));
 		return -1;
 	}
@@ -556,7 +613,7 @@ int do_insert(uint64_t user_idnr, char *name, char *source)
 	res = read_from_stream(file, &buf, -1);
 	fclose(file);
 	if (res != 0) {
-		qerrorf("Error reading in your script!\n");
+		qprintf("Error reading in your script!\n");
 		return -1;
 	}
 
@@ -564,7 +621,7 @@ int do_insert(uint64_t user_idnr, char *name, char *source)
 	res = dm_sievescript_add(user_idnr, "@!temp-script!@", buf);
 	g_free(buf);
 	if (res != 0) {
-		qerrorf("Error inserting temporary script into the database!\n");
+		qprintf("Error importing temporary script into the database!\n");
 		return -1;
 	}
 
@@ -589,7 +646,7 @@ int do_insert(uint64_t user_idnr, char *name, char *source)
 		dm_sievescript_delete(user_idnr, "@!temp-script!@");
 		return -1;
 	} else if (res != 0) {
-		qerrorf("Error inserting script [%s] into the database!\n",
+		qprintf("Error importing script [%s] into the database!\n",
 		       name);
 		dm_sievescript_delete(user_idnr, "@!temp-script!@");
 		return -1;
@@ -602,7 +659,7 @@ int do_insert(uint64_t user_idnr, char *name, char *source)
 			qprintf("Script [%s] successfully updated and remains inactive!\n", name);
 		}
 	} else {
-		qprintf("Script [%s] successfully inserted and marked inactive!\n", name);
+		qprintf("Script [%s] successfully imported and marked inactive!\n", name);
 	}
 	return 0;
 }
@@ -611,7 +668,7 @@ int do_insert(uint64_t user_idnr, char *name, char *source)
 int do_remove(uint64_t user_idnr, char *name)
 {
 	if (! dm_sievescript_delete(user_idnr, name)) {
-		qerrorf("Error deleting script [%s].\n", name);
+		qprintf("Error deleting script [%s].\n", name);
 		return -1;
 	}
 
@@ -626,7 +683,7 @@ int do_list(uint64_t user_idnr)
 	GList *scriptlist = NULL;
 
 	if (dm_sievescript_list(user_idnr, &scriptlist) < 0) {
-		qerrorf("Error retrieving Sieve script list.\n");
+		qprintf("Error retrieving Sieve script list.\n");
 		return -1;
 	}
 
@@ -653,45 +710,3 @@ int do_list(uint64_t user_idnr)
 
 	return 0;
 }
-
-
-int do_showhelp(void)
-{
-	printf(
-	"*** dbmail-sievecmd ***\n"
-//	Try to stay under the standard 80 column width
-//	0........10........20........30........40........50........60........70........80
-	"Use this program to manage your users' Sieve scripts.\n"
-	"See the man page for more info. Summary:\n\n"
-	"     -u username            Username of script user \n"
-	"     -l                     List scripts belonging to user \n"
-	"     -a scriptname          Activate the named script \n"
-	"                            (only one script can be active; \n"
-	"                             deactivates any others) \n"
-	"     -d [scriptname]        Deactivate the named script \n"
-	"     -c [scriptname]        Print the contents of the named script\n"
-	"     -e [scriptname]        Edit the contents of the named script\n"
-	"                            (if no script is given, the active \n"
-	"                             script is printed) \n"
-	"     -i scriptname file     Insert the named script from file \n"
-	"                            (a single dash, -, indicates input \n"
-	"                             from STDIN) \n"
-	"     -r scriptname          Remove the named script \n"
-	"                            (if script was active, no script is \n"
-	"                             active after deletion) \n"
-
-        "\nCommon options for all DBMail utilities:\n"
-	"     -f file   specify an alternative config file\n"
-	"     -q        quietly skip interactive prompts\n"
-	"               use twice to suppress error messages\n"
-	"     -n        show the intended action but do not perform it, no to all\n"
-	"     -y        perform all proposed actions, as though yes to all\n"
-	"     -v        verbose details\n"
-	"     -V        show the version\n"
-	"     -h        show this help message\n"
-	);
-
-	return 0;
-}
-
-
